@@ -1,5 +1,5 @@
 //
-// $Id: DiffTest.cpp 4129 2012-11-20 00:05:37Z chambm $
+// $Id: DiffTest.cpp 5759 2014-02-19 22:26:29Z chambm $
 //
 //
 // Original author: Robert Burke <robert.burke@proteowizard.org>
@@ -612,7 +612,7 @@ void testSourceFile()
     a.externalFormatDocumentation.push_back("The Idiot's Guide to External Formats");
     b.externalFormatDocumentation.push_back("External Formats for Dummies");
     a.set(MS_sample_number);
-    b.set(MS_sample_name);
+    b.set(MS_sample_name_OBSOLETE);
 
     diff(a,b);
     if (os_) *os_ << diff_string<TextWriter>(diff) << endl;
@@ -633,7 +633,7 @@ void testSourceFile()
     unit_assert(diff.a_b.cvParams.size() == 1);
     unit_assert(diff.b_a.cvParams.size() == 1);
     unit_assert(diff.a_b.hasCVParam(MS_sample_number));
-    unit_assert(diff.b_a.hasCVParam(MS_sample_name));
+    unit_assert(diff.b_a.hasCVParam(MS_sample_name_OBSOLETE));
 
 }
 
@@ -927,7 +927,7 @@ void testSpectrumIdentificationProtocol()
     a.analysisSoftwarePtr = AnalysisSoftwarePtr(new AnalysisSoftware("a_as"));
 
     a.searchType.cvid = MS_pmf_search;
-    a.additionalSearchParams.set(MS_Sequest_CleavesAt, "cleavage");
+    a.additionalSearchParams.set(MS_SEQUEST_CleavesAt, "cleavage");
     SearchModificationPtr smp(new SearchModification());
     smp->fixedMod = true;
     a.modificationParams.push_back(smp);
@@ -957,7 +957,7 @@ void testSpectrumIdentificationProtocol()
     b.analysisSoftwarePtr = AnalysisSoftwarePtr(new AnalysisSoftware("b_as"));
 
     b.searchType.cvid = MS_ms_ms_search;
-    b.additionalSearchParams.set(MS_Sequest_CleavesAt, "land");
+    b.additionalSearchParams.set(MS_SEQUEST_CleavesAt, "land");
     b.modificationParams.clear();
     b.enzymes.enzymes.clear();
     b.massTable.clear();
@@ -1162,7 +1162,7 @@ void testSample()
     unit_assert(!diff);
 
     b.contactRole.back().reset(new ContactRole(MS_role_type, ContactPtr(new Person("fer_rehto"))));
-    b.set(MS_sample_name);
+    b.set(MS_sample_name_OBSOLETE);
 
     diff(a, b);
     if (os_) *os_ << diff_string<TextWriter>(diff) << endl;
@@ -1181,7 +1181,7 @@ void testSample()
     unit_assert(diff.b_a.contactRole.back()->contactPtr.get());
     unit_assert(diff.a_b.contactRole.back()->contactPtr->id == "contactPtr");
     unit_assert(diff.b_a.contactRole.back()->contactPtr->id == "fer_rehto");
-    unit_assert(diff.b_a.hasCVParam(MS_sample_name));
+    unit_assert(diff.b_a.hasCVParam(MS_sample_name_OBSOLETE));
 }
 
 
