@@ -1,5 +1,5 @@
 //
-// $Id: Pep2MzIdent.cpp 3397 2012-03-19 19:20:57Z pcbrefugee $
+// $Id: Pep2MzIdent.cpp 5759 2014-02-19 22:26:29Z chambm $
 //
 // Original author: Robert Burke <robert.burke@proteowizard.org>
 //
@@ -900,7 +900,7 @@ CVParam guessThreshold(const vector<AnalysisSoftwarePtr>& software)
 {
     static const CVID cvids[][2] = {
         {MS_Mascot, MS_Mascot_score},
-        {MS_Sequest, MS_Sequest_probability},
+        {MS_SEQUEST, MS_SEQUEST_probability},
         {MS_Phenyx, MS_Phenyx_Score},
         {CVID_Unknown, CVID_Unknown}
     };
@@ -934,7 +934,7 @@ CVParam guessThreshold(const vector<AnalysisSoftwarePtr>& software)
 AnalysisSoftwarePtr guessAnalysisSoftware(
     const vector<AnalysisSoftwarePtr>& software)
 {
-    static const CVID cvids[] = {MS_Mascot, MS_Sequest,
+    static const CVID cvids[] = {MS_Mascot, MS_SEQUEST,
                                  MS_Phenyx, CVID_Unknown};
     AnalysisSoftwarePtr asp(new AnalysisSoftware());
 
@@ -977,38 +977,38 @@ bool fileExtension2Type(const string& file,
                         CVID& spectrumIDFormat)
 {
     // TODO map the following file formats
-    //{MS_Waters_raw_file, MS_mass_spectrometer_file_format},
+    //{MS_Waters_raw_format, MS_mass_spectrometer_file_format},
     //{MS_mass_spectrometer_file_format, MS_file_format},
-    //{MS_Thermo_RAW_file, MS_mass_spectrometer_file_format},
-    //{MS_PSI_mzData_file, MS_mass_spectrometer_file_format},
-    //{MS_Bruker_Agilent_YEP_file, MS_mass_spectrometer_file_format},
-    //{MS_ProteinLynx_Global_Server_mass_spectrum_XML_file, MS_mass_spectrometer_file_format},
-    //{MS_parameter_file, MS_mass_spectrometer_file_format},
-    //{MS_Bruker_U2_file, MS_mass_spectrometer_file_format},
-    //{MS_Sciex_API_III_file, MS_mass_spectrometer_file_format},
-    //{MS_Bruker_XML_file, MS_mass_spectrometer_file_format},
-    //{MS_text_file, MS_mass_spectrometer_file_format},
+    //{MS_Thermo_RAW_format, MS_mass_spectrometer_file_format},
+    //{MS_PSI_mzData_format, MS_mass_spectrometer_file_format},
+    //{MS_Bruker_Agilent_YEP_format, MS_mass_spectrometer_file_format},
+    //{MS_ProteinLynx_Global_Server_mass_spectrum_XML_format, MS_mass_spectrometer_file_format},
+    //{MS_parameter_format, MS_mass_spectrometer_file_format},
+    //{MS_Bruker_U2_format, MS_mass_spectrometer_file_format},
+    //{MS_Sciex_API_III_format, MS_mass_spectrometer_file_format},
+    //{MS_Bruker_XML_format, MS_mass_spectrometer_file_format},
+    //{MS_text_format, MS_mass_spectrometer_file_format},
     //{MS_Phenyx_XML_format, MS_mass_spectrometer_file_format},
     //{MS_AB_SCIEX_TOF_TOF_database, MS_mass_spectrometer_file_format},
-    //{MS_Agilent_MassHunter_file, MS_mass_spectrometer_file_format},
+    //{MS_Agilent_MassHunter_format, MS_mass_spectrometer_file_format},
     //{MS_Proteinscape_spectra, MS_mass_spectrometer_file_format},
-    //{MS_AB_SCIEX_TOF_TOF_T2D_file, MS_mass_spectrometer_file_format},
+    //{MS_AB_SCIEX_TOF_TOF_T2D_format, MS_mass_spectrometer_file_format},
 
     static fileInfo pairs[] = {
-        fileInfo(".mzml", MS_mzML_file, MS_mzML_unique_identifier),
-        fileInfo(".mzxml", MS_ISB_mzXML_file, MS_scan_number_only_nativeID_format),
-        fileInfo(".pkl", MS_Micromass_PKL_file, MS_no_nativeID_format),
-        fileInfo(".pks", MS_PerSeptive_PKS_file, MS_mass_spectrometer_file_format),
-        fileInfo(".dta", MS_DTA_file, MS_mass_spectrometer_file_format),
-        fileInfo(".srf", MS_Bioworks_SRF_file, MS_mass_spectrometer_file_format),
-        fileInfo(".baf", MS_Bruker_BAF_file, MS_mass_spectrometer_file_format),
-        fileInfo(".fid", MS_Bruker_FID_file, MS_mass_spectrometer_file_format),
-        fileInfo(".mgf", MS_Mascot_MGF_file, MS_mass_spectrometer_file_format),
-        fileInfo(".wiff", MS_ABI_WIFF_file, MS_mass_spectrometer_file_format),
+        fileInfo(".mzml", MS_mzML_format, MS_mzML_unique_identifier),
+        fileInfo(".mzxml", MS_ISB_mzXML_format, MS_scan_number_only_nativeID_format),
+        fileInfo(".pkl", MS_Micromass_PKL_format, MS_no_nativeID_format),
+        fileInfo(".pks", MS_PerSeptive_PKS_format, MS_mass_spectrometer_file_format),
+        fileInfo(".dta", MS_DTA_format, MS_mass_spectrometer_file_format),
+        fileInfo(".srf", MS_Bioworks_SRF_format, MS_mass_spectrometer_file_format),
+        fileInfo(".baf", MS_Bruker_BAF_format, MS_mass_spectrometer_file_format),
+        fileInfo(".fid", MS_Bruker_FID_format, MS_mass_spectrometer_file_format),
+        fileInfo(".mgf", MS_Mascot_MGF_format, MS_mass_spectrometer_file_format),
+        fileInfo(".wiff", MS_ABI_WIFF_format, MS_mass_spectrometer_file_format),
         //fileInfo("", CVID_Unknown,CVID_Unknown)
 
         // Need to find a differentiator between thermo & waters
-        //fileInfo(".raw", MS_Thermo_RAW_file, MS_mass_spectrometer_file_format),
+        //fileInfo(".raw", MS_Thermo_RAW_format, MS_mass_spectrometer_file_format),
     };
 
     bool found = false;
@@ -1215,7 +1215,7 @@ void Pep2MzIdent::Impl::translateSearch(const SearchSummaryPtr summary,
     SourceFilePtr sourceFile(new SourceFile());
     sourceFile->id = "SF_1";
     sourceFile->location = summary->baseName;
-    sourceFile->fileFormat.set(MS_ISB_mzXML_file);
+    sourceFile->fileFormat.set(MS_ISB_mzXML_format);
 
     result->dataCollection.inputs.sourceFile.push_back(sourceFile);
     */
@@ -1275,11 +1275,11 @@ void Pep2MzIdent::Impl::translateSearch(const SearchSummaryPtr summary,
     /*
     if (iends_with(summary->baseName, ".dat") &&
         iequals(summary->searchEngine, "Mascot"))
-        searchDatabase->fileFormat.set(MS_Mascot_DAT_file);
+        searchDatabase->fileFormat.set(MS_Mascot_DAT_format);
     else if (iequals(summary->searchEngine, "Sequest"))
-        searchDatabase->fileFormat.set(MS_Sequest_out_file);
+        searchDatabase->fileFormat.set(MS_SEQUEST_out_format);
     else if (iequals(summary->searchEngine, "X_Tandem"))
-        searchDatabase->fileFormat.set(MS_X_Tandem_xml_file);
+        searchDatabase->fileFormat.set(MS_X_Tandem_xml_format);
     */
     
     mzid->dataCollection.inputs.searchDatabase.push_back(searchDatabase);
@@ -1989,7 +1989,7 @@ CVID Pep2MzIdent::Impl::getCVID(const string& name)
         }
         else if (iequals(name, "Sequest"))
         {
-            id = MS_Sequest;
+            id = MS_SEQUEST;
         }
         else if (iequals(name, "phenyx"))
         {
