@@ -1,5 +1,5 @@
 //
-// $Id: Serializer_pepXML.cpp 4205 2012-12-14 17:03:01Z chambm $
+// $Id: Serializer_pepXML.cpp 6478 2014-07-08 20:01:38Z chambm $
 //
 //
 // Original author: Matt Chambers <matt.chambers .@. vanderbilt.edu>
@@ -80,8 +80,8 @@ struct AnalysisSoftwareTranslation
 
 const AnalysisSoftwareTranslation analysisSoftwareTranslationTable[] =
 {
-    {MS_pwiz, "ProteoWizard"},
-    {MS_Sequest, "Sequest"},
+    {MS_ProteoWizard_software, "ProteoWizard software;ProteoWizard"},
+    {MS_SEQUEST, "Sequest"},
     {MS_Mascot, "Mascot"},
     {MS_OMSSA, "OMSSA"},
     {MS_Phenyx, "Phenyx"},
@@ -150,8 +150,8 @@ struct ScoreTranslation
 
 const ScoreTranslation scoreTranslationTable[] =
 {
-    {MS_Sequest, MS_Sequest_xcorr, "xcorr"},
-    {MS_Sequest, MS_Sequest_deltacn, "deltacn;deltcn"},
+    {MS_SEQUEST, MS_SEQUEST_xcorr, "xcorr"},
+    {MS_SEQUEST, MS_SEQUEST_deltacn, "deltacn;deltcn"},
     {MS_Mascot, MS_Mascot_score, "ionscore;score"},
     {MS_Mascot, MS_Mascot_identity_threshold, "identityscore"},
     {MS_Mascot, MS_Mascot_homology_threshold, "homologyscore"},
@@ -259,11 +259,21 @@ struct NativeIdTranslator : public boost::singleton<NativeIdTranslator>
                 continue;
 
             string format = cvTermInfo(cvid).def;
-            bal::replace_all(format, "xsd:nonNegativeInteger", "\\d+");
-            bal::replace_all(format, "xsd:positiveInteger", "\\d+");
-            bal::replace_all(format, "xsd:Long", "\\d+");
-            bal::replace_all(format, "xsd:string", "\\S+");
-            bal::replace_all(format, "xsd:IDREF", "\\S+");
+            if (!bal::icontains(format, "xsd"))
+                continue;
+
+            sregex nativeIdFormatRegex = sregex::compile(".*?(\\S+=\\S+( \\S+=\\S+)*)\\.?");
+            smatch what;
+            if (!regex_match(format, what, nativeIdFormatRegex))
+                continue;
+
+            format = what[1].str();
+            bal::trim_right_if(format, bal::is_any_of("."));
+            bal::ireplace_all(format, "xsd:nonNegativeInteger", "\\d+");
+            bal::ireplace_all(format, "xsd:positiveInteger", "\\d+");
+            bal::ireplace_all(format, "xsd:Long", "\\d+");
+            bal::ireplace_all(format, "xsd:string", "\\S+");
+            bal::ireplace_all(format, "xsd:IDREF", "\\S+");
             nativeIdRegexAndFormats.push_back(make_pair(sregex::compile(format), cvid));
         }
     }
@@ -500,14 +510,21 @@ void write_search_summary(XMLWriter& xmlWriter, const IdentData& mzid, const str
         BOOST_FOREACH(const SearchModificationPtr& sm, sip.modificationParams)
         {
             vector<char> residues = sm->residues;
-            if (residues.empty())
+            if (residues.empty() || (residues.size() == 1 && residues[0] == '.'))
             {
+                residues.clear();
+
                 if (sm->specificityRules.empty())
-                    throw runtime_error("[write_search_summary] Empty SearchModification.");
-                if (sm->specificityRules == MS_modification_specificity_N_term)
-                    residues.push_back('n');
-                else
-                    residues.push_back('c');
+                    throw runtime_error("[write_search_summary] Empty or non-specific SearchModification.");
+
+                switch (sm->specificityRules.cvid)
+                {
+                    case MS_modification_specificity_peptide_N_term: residues.push_back('n'); break;
+                    case MS_modification_specificity_protein_N_term: residues.push_back('o'); break;
+                    case MS_modification_specificity_peptide_C_term: residues.push_back('c'); break;
+                    case MS_modification_specificity_protein_C_term: residues.push_back('d'); break;
+                    default: throw runtime_error("[write_search_summary] Cannot handle CV term in specificityRules: " + sm->specificityRules.name());
+                }
             }
 
             BOOST_FOREACH(char aa, residues)
@@ -515,13 +532,26 @@ void write_search_summary(XMLWriter& xmlWriter, const IdentData& mzid, const str
                 attributes.clear();
                 if (aa > 'Z') // terminal_modification
                 {
-                    attributes.add("terminus", string(1, aa));
-                    attributes.add("massdiff", sm->massDelta);
+                    switch (aa)
+                    {
+                        case 'o':
+                            attributes.add("protein_terminus", "n");
+                        case 'n':
+                            attributes.add("terminus", "n");
+                            attributes.add("massdiff", sm->massDelta);
+                            attributes.add("mass", nTerm.monoisotopicMass() + sm->massDelta);
+                            break;
 
-                    if (aa == 'n')
-                        attributes.add("mass", nTerm.monoisotopicMass() + sm->massDelta);
-                    else
-                        attributes.add("mass", cTerm.monoisotopicMass() + sm->massDelta);
+                        case 'd':
+                            attributes.add("protein_terminus", "c");
+                        case 'c':
+                            attributes.add("terminus", "c");
+                            attributes.add("massdiff", sm->massDelta);
+                            attributes.add("mass", cTerm.monoisotopicMass() + sm->massDelta);
+                            break;
+
+                        default: throw runtime_error("[write_search_summary] Unsupported AA residue: " + aa);
+                    }
                 }
                 else // aminoacid_modificiation
                 {
@@ -529,10 +559,14 @@ void write_search_summary(XMLWriter& xmlWriter, const IdentData& mzid, const str
                     attributes.add("aminoacid", string(1, aa));
                     attributes.add("massdiff", sm->massDelta);
                     attributes.add("mass", sm->massDelta + aaMass);
-                    if (sm->specificityRules == MS_modification_specificity_N_term)
-                        attributes.add("peptide_terminus", "n");
-                    if (sm->specificityRules == MS_modification_specificity_C_term)
-                        attributes.add("peptide_terminus", "c");
+                    
+                    if (!sm->specificityRules.empty())
+                        switch (sm->specificityRules.cvid)
+                        {
+                            case MS_modification_specificity_peptide_N_term: attributes.add("peptide_terminus", "n"); break;
+                            case MS_modification_specificity_peptide_C_term: attributes.add("peptide_terminus", "c"); break;
+                            default: throw runtime_error("[write_search_summary] Cannot handle CV term in specificityRules: " + sm->specificityRules.name());
+                        }
                 }
                 attributes.add("variable", sm->fixedMod ? "N" : "Y");
 
@@ -689,7 +723,7 @@ void write_search_hit(XMLWriter& xmlWriter,
         {
             if (cvParam.cvid != MS_number_of_matched_peaks &&
                 cvParam.cvid != MS_number_of_unmatched_peaks &&
-                (cvIsA(cvParam.cvid, MS_search_engine_specific_score_for_peptides) ||
+                (cvIsA(cvParam.cvid, MS_search_engine_specific_score_for_PSMs) ||
                  regex_match(cvParam.value, what, numericRegex)))
             {
                 const string& preferredScoreName = ScoreTranslator::instance->translate(analysisSoftwareCVID, cvParam.cvid);
@@ -760,7 +794,14 @@ void write_spectrum_queries(XMLWriter& xmlWriter, const IdentData& mzid, const s
 
                 string scanNumber = msdata::id::translateNativeIDToScanNumber(nativeIdFormat, sir.spectrumID);
                 if (scanNumber.empty())
-                    scanNumber = lexical_cast<string>(spectrumIndex);
+                {
+                    if (sir.hasCVParam(MS_peak_list_scans))
+                        scanNumber = sir.cvParam(MS_peak_list_scans).value;
+                    else if (sir.hasCVParam(MS_scan_number_s__OBSOLETE))
+                        scanNumber = sir.cvParam(MS_scan_number_s__OBSOLETE).value;
+                    else
+                        scanNumber = lexical_cast<string>(spectrumIndex);
+                }
 
                 // basename.scanNumber.scanNumber.charge
                 ostringstream conventionalSpectrumId;
@@ -780,8 +821,10 @@ void write_spectrum_queries(XMLWriter& xmlWriter, const IdentData& mzid, const s
 
                 if (sir.hasCVParam(MS_scan_start_time))
                     attributes.add("retention_time_sec", sir.cvParam(MS_scan_start_time).timeInSeconds());
-                else if (sir.hasCVParam(MS_retention_time_s_))
-                    attributes.add("retention_time_sec", sir.cvParam(MS_retention_time_s_).timeInSeconds());
+                else if (sir.hasCVParam(MS_retention_time))
+                    attributes.add("retention_time_sec", sir.cvParam(MS_retention_time).timeInSeconds());
+                else if (sir.hasCVParam(MS_retention_time_s__OBSOLETE))
+                    attributes.add("retention_time_sec", sir.cvParam(MS_retention_time_s__OBSOLETE).timeInSeconds());
 
                 xmlWriter.startElement("spectrum_query", attributes);
 
@@ -962,7 +1005,7 @@ struct HandlerSearchSummary : public SAXParser::Handler
             const string* preferredName;
             if (result == CVID_Unknown)
             {
-                result = MS_analysis_software;
+                result = MS_custom_unreleased_software_tool;
                 preferredName = &name;
             }
             else
@@ -970,8 +1013,8 @@ struct HandlerSearchSummary : public SAXParser::Handler
             software.reset(new AnalysisSoftware("AS_" + *preferredName, *preferredName));
             
             // TODO if MS_analysis_software log warning that search engine could not be translated
-            if (result == MS_analysis_software)
-                software->softwareName.set(MS_analysis_software, *preferredName);
+            if (result == MS_custom_unreleased_software_tool)
+                software->softwareName.set(MS_custom_unreleased_software_tool, *preferredName);
             else
                 software->softwareName.set(result);
 
@@ -999,31 +1042,21 @@ struct HandlerSearchSummary : public SAXParser::Handler
         bal::split(tokens, ionSeriesList, bal::is_any_of(","));
         BOOST_FOREACH(const string& ionSeries, tokens)
         {
-            if (ionSeries == "immonium")        _sip->additionalSearchParams.set(MS_param__immonium_ion);
-            else if (ionSeries == "a")          _sip->additionalSearchParams.set(MS_param__a_ion);
-            else if (ionSeries == "a-NH3")      _sip->additionalSearchParams.set(MS_param__a_ion_NH3);
-            else if (ionSeries == "a-H2O")      _sip->additionalSearchParams.set(MS_param__a_ion_H2O);
-            else if (ionSeries == "b")          _sip->additionalSearchParams.set(MS_param__b_ion);
-            else if (ionSeries == "b-NH3")      _sip->additionalSearchParams.set(MS_param__b_ion_NH3);
-            else if (ionSeries == "b-H2O")      _sip->additionalSearchParams.set(MS_param__b_ion_H2O);
-            else if (ionSeries == "c")          _sip->additionalSearchParams.set(MS_param__c_ion);
-            //else if (ionSeries == "c-NH3")      _sip->additionalSearchParams.set(MS_param__c_ion_NH3);
-            //else if (ionSeries == "c-H2O")      _sip->additionalSearchParams.set(MS_param__c_ion_H2O);
-            else if (ionSeries == "x")          _sip->additionalSearchParams.set(MS_param__x_ion);
-            //else if (ionSeries == "x-NH3")      _sip->additionalSearchParams.set(MS_param__x_ion_NH3);
-            //else if (ionSeries == "x-H2O")      _sip->additionalSearchParams.set(MS_param__x_ion_H2O);
-            else if (ionSeries == "y")          _sip->additionalSearchParams.set(MS_param__y_ion);
-            else if (ionSeries == "y-NH3")      _sip->additionalSearchParams.set(MS_param__y_ion_NH3);
-            else if (ionSeries == "y-H2O")      _sip->additionalSearchParams.set(MS_param__y_ion_H2O);
-            else if (ionSeries == "z")          _sip->additionalSearchParams.set(MS_param__z_ion);
-            //else if (ionSeries == "z-NH3")      _sip->additionalSearchParams.set(MS_param__z_ion_NH3);
-            //else if (ionSeries == "z-H2O")      _sip->additionalSearchParams.set(MS_param__z_ion_H2O);
+            if (ionSeries == "immonium")               _sip->additionalSearchParams.set(MS_param__immonium_ion);
+            else if (bal::contains(ionSeries, "NH3"))  _sip->additionalSearchParams.set(MS_NH3_neutral_loss);
+            else if (bal::contains(ionSeries, "H2O"))  _sip->additionalSearchParams.set(MS_H2O_neutral_loss);
+            else if (ionSeries == "a")                 _sip->additionalSearchParams.set(MS_param__a_ion);
+            else if (ionSeries == "b")                 _sip->additionalSearchParams.set(MS_param__b_ion);
+            else if (ionSeries == "c")                 _sip->additionalSearchParams.set(MS_param__c_ion);
+            else if (ionSeries == "x")                 _sip->additionalSearchParams.set(MS_param__x_ion);
+            else if (ionSeries == "y")                 _sip->additionalSearchParams.set(MS_param__y_ion);
+            else if (ionSeries == "z")                 _sip->additionalSearchParams.set(MS_param__z_ion);
             else if (ionSeries == "z+1" ||
-                     ionSeries == "z*")         _sip->additionalSearchParams.set(MS_param__z_1_ion);
-            else if (ionSeries == "z+2")        _sip->additionalSearchParams.set(MS_param__z_2_ion);
-            else if (ionSeries == "d")          _sip->additionalSearchParams.set(MS_param__d_ion);
-            else if (ionSeries == "v")          _sip->additionalSearchParams.set(MS_param__v_ion);
-            else if (ionSeries == "w")          _sip->additionalSearchParams.set(MS_param__w_ion);
+                     ionSeries == "z*")                _sip->additionalSearchParams.set(MS_param__z_1_ion);
+            else if (ionSeries == "z+2")               _sip->additionalSearchParams.set(MS_param__z_2_ion);
+            else if (ionSeries == "d")                 _sip->additionalSearchParams.set(MS_param__d_ion);
+            else if (ionSeries == "v")                 _sip->additionalSearchParams.set(MS_param__v_ion);
+            else if (ionSeries == "w")                 _sip->additionalSearchParams.set(MS_param__w_ion);
         }
     }
 
@@ -1265,9 +1298,9 @@ struct HandlerSearchSummary : public SAXParser::Handler
                 searchModification->fixedMod = lexical_cast<bool>(variable);
 
             if (bal::icontains(peptideTerminus, "n"))
-                searchModification->specificityRules.cvid = MS_modification_specificity_N_term;
+                searchModification->specificityRules.cvid = MS_modification_specificity_peptide_N_term;
             else if (bal::icontains(peptideTerminus, "c"))
-                searchModification->specificityRules.cvid = MS_modification_specificity_C_term;
+                searchModification->specificityRules.cvid = MS_modification_specificity_peptide_C_term;
 
             _sip->modificationParams.push_back(searchModification);
 
@@ -1275,26 +1308,40 @@ struct HandlerSearchSummary : public SAXParser::Handler
             if (peptideTerminus == "nc")
             {
                 searchModification.reset(new SearchModification(*searchModification));
-                searchModification->specificityRules.cvid = MS_modification_specificity_C_term;
+                searchModification->specificityRules.cvid = MS_modification_specificity_peptide_C_term;
                 _sip->modificationParams.push_back(searchModification);
             }
         }
         else if (name == "terminal_modification")
         {
-            string terminus, variable;
+            string terminus, variable, proteinTerminus;
             bal::to_lower(getAttribute(attributes, "terminus", terminus));
             bal::to_lower(getAttribute(attributes, "variable", variable));
+            bal::to_lower(getAttribute(attributes, "protein_terminus", proteinTerminus));
             
             SearchModificationPtr searchModification(new SearchModification);
             getAttribute(attributes, "massdiff", searchModification->massDelta);
             searchModification->fixedMod = !(variable == "y" || lexical_cast<bool>(variable));
-
-            if (terminus == "n")
-                searchModification->specificityRules.cvid = MS_modification_specificity_N_term;
+            searchModification->residues.push_back('.');
+            
+            if (bal::icontains(proteinTerminus, "n"))
+                searchModification->specificityRules.cvid = MS_modification_specificity_protein_N_term;
+            else if (bal::icontains(proteinTerminus, "c"))
+                searchModification->specificityRules.cvid = MS_modification_specificity_protein_C_term;
+            else if (terminus == "n")
+                searchModification->specificityRules.cvid = MS_modification_specificity_peptide_N_term;
             else if (terminus == "c")
-                searchModification->specificityRules.cvid = MS_modification_specificity_C_term;
+                searchModification->specificityRules.cvid = MS_modification_specificity_peptide_C_term;
 
             _sip->modificationParams.push_back(searchModification);
+
+            // in the case of either terminus, duplicate the mod with C terminal specificity
+            if (proteinTerminus == "nc")
+            {
+                searchModification.reset(new SearchModification(*searchModification));
+                searchModification->specificityRules.cvid = MS_modification_specificity_peptide_C_term;
+                _sip->modificationParams.push_back(searchModification);
+            }
         }
         else if (name == "parameter")
         {
@@ -1614,7 +1661,7 @@ struct HandlerSearchResults : public SAXParser::Handler
                 }
 
                 sir.reset(new SpectrumIdentificationResult);
-                sir->id = "SIR_" + lexical_cast<string>(_sil->spectrumIdentificationResult.size());
+                sir->id = "SIR_" + lexical_cast<string>(_sil->spectrumIdentificationResult.size()+1);
                 sir->spectrumID = spectrumNativeID;
                 sir->name = spectrumWithoutCharge;
                 sir->spectraDataPtr = _mzid->dataCollection.inputs.spectraData[0];
@@ -1746,7 +1793,7 @@ struct Handler_pepXML : public SAXParser::Handler
             const string* preferredName;
             if (result == CVID_Unknown)
             {
-                result = MS_analysis_software;
+                result = MS_custom_unreleased_software_tool;
                 preferredName = &name;
             }
             else
@@ -1758,8 +1805,8 @@ struct Handler_pepXML : public SAXParser::Handler
             software->name = *preferredName;
 
             // TODO if MS_analysis_software log warning that search engine could not be translated
-            if (result == MS_analysis_software)
-                software->softwareName.set(MS_analysis_software, *preferredName);
+            if (result == MS_custom_unreleased_software_tool)
+                software->softwareName.set(MS_custom_unreleased_software_tool, *preferredName);
             else
                 software->softwareName.set(result);
 
@@ -1819,11 +1866,11 @@ struct Handler_pepXML : public SAXParser::Handler
     }
 
     private:
+    CVTranslator cvTranslator;
     HandlerSampleEnzyme handlerSampleEnzyme;
     HandlerSearchSummary handlerSearchSummary;
     HandlerSearchResults handlerSearchResults;
 
-    CVTranslator cvTranslator;
     bool readSpectrumQueries;
     const IterationListenerRegistry* ilr;
     bool strict;
