@@ -1,5 +1,5 @@
 //
-// $Id: Serializer_protXML.cpp 3703 2012-06-15 18:23:36Z pcbrefugee $
+// $Id: Serializer_protXML.cpp 4922 2013-09-05 22:33:08Z pcbrefugee $
 //
 //
 // Original author: Brian Pratt <brian.pratt .@. insilicos.com>
@@ -230,7 +230,7 @@ struct Handler_protXML : public SAXParser::Handler
             pdh = ProteinDetectionHypothesisPtr(new ProteinDetectionHypothesis(protein_name));
             GET_TYPED_ATTR(string, probability);
             GET_TYPED_ATTR(string, percent_coverage);
-            pdh->set(pwiz::cv::MS_search_engine_specific_score_for_peptides, probability); // as in OpenMS
+            pdh->set(pwiz::cv::MS_search_engine_specific_score_for_PSMs, probability); // as in OpenMS
             pdh->set(pwiz::cv::MS_sequence_coverage, percent_coverage);
 
             pagPtr->proteinDetectionHypothesis.push_back(pdh);
@@ -445,7 +445,7 @@ struct Handler_protXML : public SAXParser::Handler
                 pe->dbSequencePtr =  dbSequence;
                 // TODO GET_TYPED_ATTR(double, initial_probability);
                 GET_TYPED_ATTR(double, nsp_adjusted_probability);
-                pe->set(MS_search_engine_specific_score_for_peptides, nsp_adjusted_probability);
+                pe->set(MS_search_engine_specific_score_for_PSMs, nsp_adjusted_probability);
                 mzid.sequenceCollection.peptideEvidence.push_back(pe);
             }
             bool pefound = false;
