@@ -1,5 +1,5 @@
 //
-// $Id: examples.cpp 4205 2012-12-14 17:03:01Z chambm $ 
+// $Id: examples.cpp 6478 2014-07-08 20:01:38Z chambm $ 
 //
 //
 // Original author: Robert Burke <robert.burke@proteowizard.org>
@@ -233,11 +233,9 @@ PWIZ_API_DECL void initializeBasicSpectrumIdentification(IdentData& mzid)
     sip->additionalSearchParams.set(MS_parent_mass_type_mono);
     sip->additionalSearchParams.set(MS_fragment_mass_type_mono);
     sip->additionalSearchParams.set(MS_param__a_ion);
-    sip->additionalSearchParams.set(MS_param__a_ion_NH3);
     sip->additionalSearchParams.set(MS_param__b_ion);
-    sip->additionalSearchParams.set(MS_param__b_ion_NH3);
     sip->additionalSearchParams.set(MS_param__y_ion);
-    sip->additionalSearchParams.set(MS_param__y_ion_NH3);
+    sip->additionalSearchParams.set(MS_NH3_neutral_loss);
     sip->additionalSearchParams.userParams.push_back(UserParam("INSTRUMENT", "Default"));
     sip->additionalSearchParams.userParams.push_back(UserParam("MASS", "Monoisotopic"));
     sip->additionalSearchParams.userParams.push_back(UserParam("PFA", "1"));
@@ -248,13 +246,14 @@ PWIZ_API_DECL void initializeBasicSpectrumIdentification(IdentData& mzid)
 
     SearchModificationPtr sm(new SearchModification);
     sm->massDelta = acetylation.monoisotopicMass();
-    sm->specificityRules.cvid = MS_modification_specificity_N_term;
+    sm->residues.push_back('.');
+    sm->specificityRules.cvid = MS_modification_specificity_protein_N_term;
     sm->set(UNIMOD_Acetyl);
     sip->modificationParams.push_back(sm);
 
     sm = SearchModificationPtr(new SearchModification);
     sm->massDelta = amidation.monoisotopicMass();
-    sm->specificityRules.cvid = MS_modification_specificity_C_term;
+    sm->specificityRules.cvid = MS_modification_specificity_peptide_C_term;
     sm->set(UNIMOD_Amidated);
     sip->modificationParams.push_back(sm);
 
@@ -266,7 +265,7 @@ PWIZ_API_DECL void initializeBasicSpectrumIdentification(IdentData& mzid)
 
     sm = SearchModificationPtr(new SearchModification);
     sm->massDelta = pyroglutQ.monoisotopicMass();
-    sm->specificityRules.cvid = MS_modification_specificity_N_term;
+    sm->specificityRules.cvid = MS_modification_specificity_peptide_N_term;
     sm->residues.push_back('Q');
     sm->set(UNIMOD_Gln__pyro_Glu);
     sip->modificationParams.push_back(sm);
@@ -351,7 +350,7 @@ PWIZ_API_DECL void initializeBasicSpectrumIdentification(IdentData& mzid)
     sd->id = "SD";
     sd->name = "tiny";
     sd->location = "file:///data/is/here/tiny.mzML";
-    sd->fileFormat.cvid = MS_mzML_file;
+    sd->fileFormat.cvid = MS_mzML_format;
     sd->spectrumIDFormat.cvid = MS_Thermo_nativeID_format;
     mzid.dataCollection.inputs.spectraData.push_back(sd);
 
@@ -360,7 +359,7 @@ PWIZ_API_DECL void initializeBasicSpectrumIdentification(IdentData& mzid)
     /*SourceFilePtr sourceFile(new SourceFile);
     sourceFile->id = "SF_1";
     sourceFile->location = "file:///../data/Mascot_mzml_example.dat";
-    sourceFile->fileFormat.set(MS_Mascot_DAT_file);
+    sourceFile->fileFormat.set(MS_Mascot_DAT_format);
     mzid.dataCollection.inputs.sourceFile.push_back(sourceFile);*/
 
     SpectrumIdentificationListPtr sil(new SpectrumIdentificationList("SIL"));
