@@ -1,5 +1,5 @@
 //
-// $Id: Serializer_MSn.cpp 4335 2013-02-11 20:09:50Z chambm $
+// $Id: Serializer_MSn.cpp 6585 2014-08-07 22:49:28Z chambm $
 //
 //
 // Original author: Barbara Frewen <ferwen@u.washington.edu>
@@ -30,6 +30,7 @@
 #include "pwiz/utility/chemistry/Chemistry.hpp"
 #include "zlib.h"
 #include <time.h>
+#include "SpectrumWorkerThreads.hpp"
 
 
 namespace pwiz {
@@ -100,7 +101,7 @@ namespace
     // if no accurate mass, compute it from mz and charge
     double calculateMass(double mz, int charge)
     {
-        return (mz * charge) - ((charge - 1)*Proton);
+        return (mz * charge) - (charge * Proton);
     }
 
     size_t getChargeStates(const SelectedIon& si, vector<int>& charges, vector<double>& masses)
@@ -437,9 +438,11 @@ void Serializer_MSn::Impl::write(ostream& os, const MSData& msd,
     // Go through the spectrum list and write each spectrum
     bool ms1File = MSn_Type_MS1 == _filetype || MSn_Type_BMS1 == _filetype || MSn_Type_CMS1 == _filetype;
     SpectrumList& sl = *msd.run.spectrumListPtr;
+    SpectrumWorkerThreads spectrumWorkers(sl);
     for (size_t i=0, end=sl.size(); i < end; ++i)
     {
-        SpectrumPtr s = sl.spectrum(i, true);
+        //SpectrumPtr s = sl.spectrum(i, true);
+        SpectrumPtr s = spectrumWorkers.processBatch(i);
         int msLevel = s->cvParam(MS_ms_level).valueAs<int>();
         if ((ms1File && msLevel == 1) ||
             (!ms1File && msLevel == 2 && !s->precursors.empty() && !s->precursors[0].selectedIons.empty()))
