@@ -1,5 +1,5 @@
 //
-// $Id: SpectrumList_MSn.cpp 4009 2012-10-17 18:40:44Z kaipot $
+// $Id: SpectrumList_MSn.cpp 6585 2014-08-07 22:49:28Z chambm $
 //
 //
 // Original author: Barbara Frewen <frewen@u.washington.edu>
@@ -28,6 +28,7 @@
 #include "pwiz/utility/misc/Std.hpp"
 #include "pwiz/utility/chemistry/Chemistry.hpp"
 #include "zlib.h"
+#include <boost/thread.hpp>
 
 
 namespace pwiz {
@@ -165,6 +166,7 @@ class SpectrumList_MSnImpl : public SpectrumList_MSn
   
   SpectrumPtr spectrum(size_t index, bool getBinaryData) const
   {
+    boost::lock_guard<boost::mutex> lock(readMutex);  // lock_guard will unlock mutex when out of scope or when exception thrown (during destruction)
     if (index > index_.size())
       throw runtime_error("[SpectrumList_MSn::spectrum] Index out of bounds");
     
@@ -199,6 +201,7 @@ class SpectrumList_MSnImpl : public SpectrumList_MSn
   map<string, size_t> idToIndex_;
   int version_; // read from fileheader for bms1, cms1, bms2, and cms2 filetypes
   MSn_Type filetype_;
+  mutable boost::mutex readMutex;
 
   void parseSpectrumText(Spectrum& spectrum, bool getBinaryData) const
   {
@@ -229,7 +232,7 @@ class SpectrumList_MSnImpl : public SpectrumList_MSn
     double basePeakMZ = 0;
     double basePeakIntensity = 0;
     spectrum.defaultArrayLength = 0;
-    spectrum.setMZIntensityArrays(vector<double>(), vector<double>(), MS_number_of_counts);
+    spectrum.setMZIntensityArrays(vector<double>(), vector<double>(), MS_number_of_detector_counts);
     vector<double>& mzArray = spectrum.getMZArray()->data;
     vector<double>& intensityArray = spectrum.getIntensityArray()->data;
     double precursor_mz = 0;
@@ -301,7 +304,18 @@ class SpectrumList_MSnImpl : public SpectrumList_MSn
             
             size_t first_space_pos = lineStr.find_first_of(" \t");
             size_t first_num_pos = lineStr.find_first_of("0123456789", first_space_pos);
-            charges.push_back(lexical_cast<int>(lineStr.substr(first_num_pos, 1)));  // assume one digit
+            size_t next_space_pos = lineStr.find_first_of(" \t", first_num_pos);
+            int charge = lexical_cast<int>(lineStr.substr(first_num_pos, next_space_pos-first_num_pos));
+            charges.push_back(charge);
+
+            size_t last_num_pos = lineStr.find_last_of("0123456789");
+            size_t last_space_pos = lineStr.find_last_of(" \t", last_num_pos);
+            double z_precursor_mz = calculateMassOverCharge(
+                lexical_cast<double>(lineStr.substr(last_space_pos, last_num_pos-last_space_pos+1)), charge, 1);
+            stringstream ss;
+            ss << charge << ' ' << z_precursor_mz;
+            // Store Z line information in UserParams, in the format "<charge> <m/z calculated from Z line mass>"
+            spectrum.userParams.push_back(UserParam("ms2 file charge state", ss.str()));
         }
         else if (lineStr.find("I") == 0)
         {
@@ -439,7 +453,7 @@ class SpectrumList_MSnImpl : public SpectrumList_MSn
   double calculateMassOverCharge(double mass, int charge, int charges_on_mass /* = 0 for neutral mass */) const
   {
     double neutralMass = mass - (charges_on_mass * Proton);
-    double mz = (neutralMass + ((charge - 1) * Proton)) / charge;
+    double mz = (neutralMass + (charge * Proton)) / charge;
     return mz;
   }
 
@@ -571,7 +585,7 @@ class SpectrumList_MSnImpl : public SpectrumList_MSn
     double basePeakMZ = 0;  // we may already have these but we have to read the peaks anyway...
     double basePeakIntensity = 0;
     spectrum.defaultArrayLength = 0;
-    spectrum.setMZIntensityArrays(vector<double>(), vector<double>(), MS_number_of_counts);
+    spectrum.setMZIntensityArrays(vector<double>(), vector<double>(), MS_number_of_detector_counts);
     vector<double>& mzArray = spectrum.getMZArray()->data;
     vector<double>& intensityArray = spectrum.getIntensityArray()->data;
     double mz = 0;
