@@ -1,5 +1,5 @@
 //
-// $Id: SpectrumList_Bruker.cpp 4623 2013-06-03 19:57:37Z chambm $
+// $Id: SpectrumList_Bruker.cpp 6478 2014-07-08 20:01:38Z chambm $
 //
 //
 // Original author: Matt Chambers <matt.chambers .@. vanderbilt.edu>
@@ -46,6 +46,7 @@ namespace pwiz {
 namespace msdata {
 namespace detail {
 
+using namespace Bruker;
 
 const char* parameterAlternativeNames[] =
 {
@@ -244,7 +245,7 @@ PWIZ_API_DECL SpectrumPtr SpectrumList_Bruker::spectrum(size_t index, DetailLeve
 
             if (detailLevel == DetailLevel_FullData)
             {
-                result->setMZIntensityArrays(vector<double>(), vector<double>(), MS_number_of_counts);
+                result->setMZIntensityArrays(vector<double>(), vector<double>(), MS_number_of_detector_counts);
                 result->defaultArrayLength = lcX.size();
 
                 BinaryDataArrayPtr mzArray = result->getMZArray();
@@ -427,10 +428,14 @@ PWIZ_API_DECL SpectrumPtr SpectrumList_Bruker::spectrum(size_t index, DetailLeve
 
         if (detailLevel == DetailLevel_FullData)
         {
-            result->setMZIntensityArrays(vector<double>(), vector<double>(), MS_number_of_counts);
+            result->setMZIntensityArrays(vector<double>(), vector<double>(), MS_number_of_detector_counts);
 	        automation_vector<double> mzArray, intensityArray;
             if (getLineData)
+            {
                 spectrum->getLineData(mzArray, intensityArray);
+                if (spectrum->hasProfileData())
+                    result->set(MS_profile_spectrum); // let SpectrumList_PeakPicker know this was a profile spectrum
+            }
             else
                 spectrum->getProfileData(mzArray, intensityArray);
             result->getMZArray()->data.assign(mzArray.begin(), mzArray.end());
@@ -503,9 +508,10 @@ PWIZ_API_DECL void SpectrumList_Bruker::fillSourceList()
                 bfs::path relativePath = sourcePaths_[i] / "fid";
                 if (rootpath_.has_branch_path())
                     relativePath = bal::replace_first_copy(relativePath.string(), rootpath_.branch_path().string() + NATIVE_PATH_SLASH, "");
+                relativePath = bal::replace_all_copy(relativePath.string(), NATIVE_PATH_SLASH, "/");
                 addSource(msd_, relativePath, rootpath_);
                 msd_.fileDescription.sourceFilePtrs.back()->set(MS_Bruker_FID_nativeID_format);
-                msd_.fileDescription.sourceFilePtrs.back()->set(MS_Bruker_FID_file);
+                msd_.fileDescription.sourceFilePtrs.back()->set(MS_Bruker_FID_format);
             }
             break;
 
@@ -517,7 +523,7 @@ PWIZ_API_DECL void SpectrumList_Bruker::fillSourceList()
                 bfs::path relativePath = bfs::path(rootpath_.filename()) / "Analysis.yep";
                 addSource(msd_, relativePath, rootpath_);
                 msd_.fileDescription.sourceFilePtrs.back()->set(MS_Bruker_Agilent_YEP_nativeID_format);
-                msd_.fileDescription.sourceFilePtrs.back()->set(MS_Bruker_Agilent_YEP_file);
+                msd_.fileDescription.sourceFilePtrs.back()->set(MS_Bruker_Agilent_YEP_format);
                 msd_.run.defaultSourceFilePtr = msd_.fileDescription.sourceFilePtrs.back();
             }
             break;
@@ -530,7 +536,7 @@ PWIZ_API_DECL void SpectrumList_Bruker::fillSourceList()
                 bfs::path relativePath = bfs::path(rootpath_.filename()) / "Analysis.baf";
                 addSource(msd_, relativePath, rootpath_);
                 msd_.fileDescription.sourceFilePtrs.back()->set(MS_Bruker_BAF_nativeID_format);
-                msd_.fileDescription.sourceFilePtrs.back()->set(MS_Bruker_BAF_file);
+                msd_.fileDescription.sourceFilePtrs.back()->set(MS_Bruker_BAF_format);
                 msd_.run.defaultSourceFilePtr = msd_.fileDescription.sourceFilePtrs.back();
             }
             break;
@@ -543,7 +549,7 @@ PWIZ_API_DECL void SpectrumList_Bruker::fillSourceList()
                 bfs::path relativePath = bfs::path(rootpath_.filename()) / "Analysis.baf";
                 addSource(msd_, relativePath, rootpath_);
                 msd_.fileDescription.sourceFilePtrs.back()->set(MS_Bruker_BAF_nativeID_format);
-                msd_.fileDescription.sourceFilePtrs.back()->set(MS_Bruker_BAF_file);
+                msd_.fileDescription.sourceFilePtrs.back()->set(MS_Bruker_BAF_format);
                 msd_.run.defaultSourceFilePtr = msd_.fileDescription.sourceFilePtrs.back();
             }
 
@@ -553,9 +559,10 @@ PWIZ_API_DECL void SpectrumList_Bruker::fillSourceList()
                 bfs::path relativePath = sourcePaths_.back();
                 if (rootpath_.has_branch_path())
                     relativePath = bal::replace_first_copy(relativePath.string(), rootpath_.branch_path().string() + NATIVE_PATH_SLASH, "");
+                relativePath = bal::replace_all_copy(relativePath.string(), NATIVE_PATH_SLASH, "/");
                 addSource(msd_, relativePath, rootpath_);
                 msd_.fileDescription.sourceFilePtrs.back()->set(MS_Bruker_U2_nativeID_format);
-                msd_.fileDescription.sourceFilePtrs.back()->set(MS_Bruker_U2_file);
+                msd_.fileDescription.sourceFilePtrs.back()->set(MS_Bruker_U2_format);
             }
             break;
 
@@ -566,9 +573,10 @@ PWIZ_API_DECL void SpectrumList_Bruker::fillSourceList()
                 bfs::path relativePath = sourcePaths_.back();
                 if (rootpath_.has_branch_path())
                     relativePath = bal::replace_first_copy(relativePath.string(), rootpath_.branch_path().string() + NATIVE_PATH_SLASH, "");
+                relativePath = bal::replace_all_copy(relativePath.string(), NATIVE_PATH_SLASH, "/");
                 addSource(msd_, relativePath, rootpath_);
                 msd_.fileDescription.sourceFilePtrs.back()->set(MS_Bruker_U2_nativeID_format);
-                msd_.fileDescription.sourceFilePtrs.back()->set(MS_Bruker_U2_file);
+                msd_.fileDescription.sourceFilePtrs.back()->set(MS_Bruker_U2_format);
                 msd_.run.defaultSourceFilePtr = msd_.fileDescription.sourceFilePtrs.back();
             }
             break;
