@@ -1,5 +1,5 @@
 //
-// $Id: SpectrumList_Thermo.cpp 4299 2013-01-25 19:55:19Z chambm $
+// $Id: SpectrumList_Thermo.cpp 6585 2014-08-07 22:49:28Z chambm $
 //
 //
 // Original author: Darren Kessner <darren@proteowizard.org>
@@ -32,6 +32,7 @@
 #include "pwiz/utility/misc/Filesystem.hpp"
 #include "pwiz/utility/misc/Std.hpp"
 #include <boost/bind.hpp>
+#include <boost/spirit/include/karma.hpp>
 
 
 using namespace pwiz::cv;
@@ -41,6 +42,7 @@ namespace pwiz {
 namespace msdata {
 namespace detail {
 
+using namespace Thermo;
 
 SpectrumList_Thermo::SpectrumList_Thermo(const MSData& msd, RawFilePtr rawfile, const Reader::Config& config)
 :   msd_(msd), rawfile_(rawfile), config_(config),
@@ -75,13 +77,10 @@ PWIZ_API_DECL size_t SpectrumList_Thermo::find(const string& id) const
     }
     catch (bad_lexical_cast&)
     {
-        try
-        {
-            size_t scanNumber = lexical_cast<size_t>(id::value(id, "scan"));
-            if (scanNumber>=1 && scanNumber<=size())
-                return scanNumber-1;
-        }
-        catch (bad_lexical_cast&) {}
+        map<string, size_t>::const_iterator scanItr = idToIndexMap_.find(id);
+        if (scanItr == idToIndexMap_.end())
+            return size_;
+        return scanItr->second;
     }
 
     return size();
@@ -150,6 +149,7 @@ PWIZ_API_DECL SpectrumPtr SpectrumList_Thermo::spectrum(size_t index, bool getBi
 
 PWIZ_API_DECL SpectrumPtr SpectrumList_Thermo::spectrum(size_t index, DetailLevel detailLevel, const pwiz::util::IntegerSet& msLevelsToCentroid) const 
 { 
+    boost::lock_guard<boost::recursive_mutex> lock(readMutex);  // lock_guard will unlock mutex when out of scope or when exception thrown (during destruction)
     if (index >= size_)
         throw runtime_error("[SpectrumList_Thermo::spectrum()] Bad index: " 
                             + lexical_cast<string>(index));
@@ -229,6 +229,11 @@ PWIZ_API_DECL SpectrumPtr SpectrumList_Thermo::spectrum(size_t index, DetailLeve
 
     try
     {
+        if (scanInfo->isConstantNeutralLoss())
+        {
+            scan.set(MS_analyzer_scan_offset, scanInfo->analyzerScanOffset(), MS_m_z);
+        }
+
         // special handling for non-MS scans
         if (ie.controllerType != Controller_MS)
         {            
@@ -250,7 +255,7 @@ PWIZ_API_DECL SpectrumPtr SpectrumList_Thermo::spectrum(size_t index, DetailLeve
             if (getBinaryData)
             {
                 result->setMZIntensityPairs(reinterpret_cast<MZIntensityPair*>(xyList->data()), 
-                                            xyList->size(), MS_number_of_counts);
+                                            xyList->size(), MS_number_of_detector_counts);
 
                 // replace "m/z array" term with "wavelength array"
                 BinaryDataArray& xArray = *result->getMZArray();
@@ -310,7 +315,7 @@ PWIZ_API_DECL SpectrumPtr SpectrumList_Thermo::spectrum(size_t index, DetailLeve
         }
 
         result->set(MS_base_peak_m_z, scanInfo->basePeakMass(), MS_m_z);
-        result->set(MS_base_peak_intensity, scanInfo->basePeakIntensity(), MS_number_of_counts);
+        result->set(MS_base_peak_intensity, scanInfo->basePeakIntensity(), MS_number_of_detector_counts);
         result->set(MS_total_ion_current, scanInfo->totalIonCurrent());
 
         if (scanInfo->FAIMSOn())
@@ -432,6 +437,8 @@ PWIZ_API_DECL SpectrumPtr SpectrumList_Thermo::spectrum(size_t index, DetailLeve
             // isolationWindow
 
             double isolationWidth = 0;
+            const double defaultIsolationWindowLowerOffset = 1.5;
+            const double defaultIsolationWindowUpperOffset = 2.5;
 
             try
             {
@@ -454,14 +461,20 @@ PWIZ_API_DECL SpectrumPtr SpectrumList_Thermo::spectrum(size_t index, DetailLeve
             if (msLevel == -1) // precursor ion scan
             {
                 product.isolationWindow.set(MS_isolation_window_target_m_z, isolationMz, MS_m_z);
-                product.isolationWindow.set(MS_isolation_window_lower_offset, isolationWidth, MS_m_z);
-                product.isolationWindow.set(MS_isolation_window_upper_offset, isolationWidth, MS_m_z);
+                if (isolationWidth != 0)
+                {
+                    product.isolationWindow.set(MS_isolation_window_lower_offset, isolationWidth, MS_m_z);
+                    product.isolationWindow.set(MS_isolation_window_upper_offset, isolationWidth, MS_m_z);
+                }
             }
             else
             {
                 precursor.isolationWindow.set(MS_isolation_window_target_m_z, isolationMz, MS_m_z);
-                precursor.isolationWindow.set(MS_isolation_window_lower_offset, isolationWidth, MS_m_z);
-                precursor.isolationWindow.set(MS_isolation_window_upper_offset, isolationWidth, MS_m_z);
+                if (isolationWidth != 0)
+                {
+                    precursor.isolationWindow.set(MS_isolation_window_lower_offset, isolationWidth, MS_m_z);
+                    precursor.isolationWindow.set(MS_isolation_window_upper_offset, isolationWidth, MS_m_z);
+                }
             }
 
             double selectedIonMz = scanInfo->precursorMZ(i); // monoisotopic m/z is preferred
@@ -497,7 +510,9 @@ PWIZ_API_DECL SpectrumPtr SpectrumList_Thermo::spectrum(size_t index, DetailLeve
                 }
 
                 // if the monoisotopic m/z is outside the isolation window (due to Thermo firmware bug), reset it to isolation m/z
-                if ((selectedIonMz < (isolationMz - isolationWidth)) || (selectedIonMz > (isolationMz + isolationWidth)))
+                if (isolationWidth == 0 && (selectedIonMz < (isolationMz-defaultIsolationWindowLowerOffset)) || (selectedIonMz > (isolationMz+defaultIsolationWindowUpperOffset)))
+                    selectedIonMz = isolationMz;
+                else if ((selectedIonMz < (isolationMz-isolationWidth)) || (selectedIonMz > (isolationMz+isolationWidth)))
                     selectedIonMz = isolationMz;
 
                 // add selected ion m/z (even if it's still equal to isolation m/z)
@@ -517,35 +532,13 @@ PWIZ_API_DECL SpectrumPtr SpectrumList_Thermo::spectrum(size_t index, DetailLeve
                 {
                     precursor.spectrumID = index_[precursorScanIndex].id;
 
-                    // retrieve the points within the isolation window
-                    /* TODO: figure out why this fails with HRESULT=6 on some files
-                    MassRangePtr massRange = MassRangePtr(new MassRange());
-                    if (isolationWidth == 0)
-                        massRange->low = isolationMz-1.5; massRange->high = isolationMz+2.5;
-                    else
-                    {
-                        massRange->low = isolationMz-isolationWidth;
-                        massRange->high = isolationMz+isolationWidth;
-                    }	
-                    MassListPtr massList = rawfile_->getMassList(precursorScanIndex, "", Cutoff_None, 0, 0, true, MassList_Current, massRange);
-                    double peakIntensity = 0;
-                    if (massList->size() > 0)
-                    {
-                        for (int i=0; i < massList->size(); ++i)
-                            peakIntensity = max(peakIntensity, massList->data()[i].intensity);
-                    }
-                    if (peakIntensity > 0)
-                        selectedIon.set(MS_peak_intensity, peakIntensity, MS_number_of_counts);
-                    */
-
                     if (detailLevel >= DetailLevel_FullMetadata)
                     {
                         // add precursor intensity
                         // retrieve the intensity of the base peak within the isolation window
-                        // TODO: determine correct window around precursor m/z
                         ostringstream massRangeStream;
                         if (isolationWidth == 0)
-                            massRangeStream << setprecision(7) << (isolationMz-1.5) << '-' << (isolationMz+2.5);
+                            massRangeStream << setprecision(7) << (isolationMz-defaultIsolationWindowLowerOffset) << '-' << (isolationMz+defaultIsolationWindowUpperOffset);
                         else
                             massRangeStream << setprecision(7) << (isolationMz-isolationWidth) << '-' << (isolationMz+isolationWidth);
                         double precursorScanTime = rawfile_->rt(index_[precursorScanIndex].scan);
@@ -559,7 +552,7 @@ PWIZ_API_DECL SpectrumPtr SpectrumList_Thermo::spectrum(size_t index, DetailLeve
                             throw runtime_error("[SpectrumList_Thermo::spectrum()] No chromatogram time points for the scan time.");
 
                         if (c->data()[0].intensity > 0)
-                            selectedIon.set(MS_peak_intensity, c->data()[0].intensity, MS_number_of_counts);
+                            selectedIon.set(MS_peak_intensity, c->data()[0].intensity, MS_number_of_detector_counts);
                     }
                 }
             }
@@ -596,10 +589,13 @@ PWIZ_API_DECL SpectrumPtr SpectrumList_Thermo::spectrum(size_t index, DetailLeve
 			{
 				// use label data for accurate centroids on FT profile data
 				massList = rawfile_->getMassListFromLabelData(ie.scan);
+                result->set(MS_profile_spectrum); // let SpectrumList_PeakPicker know this was a profile spectrum
 			}
 			else
 			{
 				massList = rawfile_->getMassList(ie.scan, "", Cutoff_None, 0, 0, doCentroid);
+                if (doCentroid)
+                    result->set(MS_profile_spectrum); // let SpectrumList_PeakPicker know this was a profile spectrum
 			}
 
 			result->defaultArrayLength = massList->size();
@@ -613,7 +609,7 @@ PWIZ_API_DECL SpectrumPtr SpectrumList_Thermo::spectrum(size_t index, DetailLeve
 			if (getBinaryData)
 			{
 				result->setMZIntensityPairs(reinterpret_cast<MZIntensityPair*>(massList->data()), 
-											massList->size(), MS_number_of_counts);
+											massList->size(), MS_number_of_detector_counts);
 			}
 		}
 
@@ -642,6 +638,8 @@ PWIZ_API_DECL int SpectrumList_Thermo::numSpectraOfMSOrder(MSOrder msOrder) cons
 
 PWIZ_API_DECL void SpectrumList_Thermo::createIndex()
 {
+    using namespace boost::spirit::karma;
+
     spectraByScanType.resize(ScanType_Count, 0);
     spectraByMSOrder.resize(MSOrder_Count+3, 0); // can't use negative index and a std::map would be inefficient
 
@@ -698,11 +696,11 @@ PWIZ_API_DECL void SpectrumList_Thermo::createIndex()
                         ie.scan = scan;
                         ie.index = index_.size()-1;
 
-                        ostringstream oss;
-                        oss << "controllerType=" << ie.controllerType <<
-                               " controllerNumber=" << ie.controllerNumber <<
-                               " scan=" << ie.scan;
-                        ie.id = oss.str();
+                        std::back_insert_iterator<std::string> sink(ie.id);
+                        generate(sink,
+                                 "controllerType=" << int_ << " controllerNumber=" << int_ << " scan=" << int_,
+                                 (int) ie.controllerType, ie.controllerNumber, ie.scan);
+                        idToIndexMap_[ie.id] = ie.index;
 
                         ie.scanType = scanType;
                         ie.msOrder = msOrder;
@@ -722,11 +720,11 @@ PWIZ_API_DECL void SpectrumList_Thermo::createIndex()
                         ie.scan = scan;
                         ie.index = index_.size()-1;
 
-                        ostringstream oss;
-                        oss << "controllerType=" << ie.controllerType <<
-                               " controllerNumber=" << ie.controllerNumber <<
-                               " scan=" << ie.scan;
-                        ie.id = oss.str();
+                        std::back_insert_iterator<std::string> sink(ie.id);
+                        generate(sink,
+                                 "controllerType=" << int_ << " controllerNumber=" << int_ << " scan=" << int_,
+                                 (int) ie.controllerType, ie.controllerNumber, ie.scan);
+                        idToIndexMap_[ie.id] = ie.index;
                     }
                 }
                 break;
