@@ -1,5 +1,5 @@
 //
-// $Id: SAXParser.cpp 4243 2012-12-28 20:24:37Z chambm $
+// $Id: SAXParser.cpp 4778 2013-08-01 20:45:09Z kaipot $
 //
 //
 // Original author: Darren Kessner <darren@proteowizard.org>
@@ -287,16 +287,20 @@ void Handler::Attributes::parseAttributes(string::size_type& index) const
                 char quoteChar = *c;
                 const char *q = strchr(textbuff+indexQuoteOpen+1,quoteChar);
                 indexQuoteClose = q?(q-textbuff):string::npos;
-            } 
+            }
             else 
                 indexQuoteClose = string::npos;
             if (indexQuoteClose == string::npos) 
             { // this index can only be OK if the others are too
-                if ('/'==textbuff[indexNameBegin]) 
+                if ('/'==textbuff[indexNameBegin])
                 { // end of tag
                     index++; 
                     break;
                 }
+                else if ('\0'==textbuff[indexNameBegin])
+                {
+                    break;
+                }
                 throw runtime_error("[SAXParser::parseAttribute()] Error at index "
                 + lexical_cast<string>(index) + ":\n" + textbuff);
             }
