@@ -1,5 +1,5 @@
 //
-// $Id: msconvert.cpp 4449 2013-03-28 20:51:00Z pcbrefugee $
+// $Id: msconvert.cpp 6312 2014-06-04 16:46:40Z chambm $
 //
 //
 // Original author: Darren Kessner <darren@proteowizard.org>
@@ -32,8 +32,10 @@
 #include "pwiz/data/msdata/Version.hpp"
 #include "pwiz/analysis/Version.hpp"
 #include "boost/program_options.hpp"
+#include "boost/format.hpp"
 #include "pwiz/utility/misc/Filesystem.hpp"
 #include "pwiz/utility/misc/Std.hpp"
+#include <boost/filesystem/detail/utf8_codecvt_facet.hpp>
 
 using namespace pwiz::cv;
 using namespace pwiz::data;
@@ -45,7 +47,7 @@ ostream* os_ = &cout;
 
 
 /// Holds the results of the parseCommandLine function. 
-struct Config
+struct Config : public Reader::Config
 {
     vector<string> filenames;
     vector<string> filters;
@@ -56,12 +58,14 @@ struct Config
     MSDataFile::WriteConfig writeConfig;
     string contactFilename;
     bool merge;
-    bool simAsSpectra;
-    bool srmAsSpectra;
 
     Config()
-    :   outputPath("."), verbose(false), merge(false), simAsSpectra(false), srmAsSpectra(false)
-    {}
+        : outputPath("."), verbose(false), merge(false)
+    {
+        simAsSpectra = false;
+        srmAsSpectra = false;
+        combineIonMobilitySpectra = false;
+    }
 
     string outputFilename(const string& inputFilename, const MSData& inputMSData) const;
 };
@@ -124,11 +128,23 @@ ostream& operator<<(ostream& os, const Config& config)
     return os;
 }
 
+static double string_to_double( const std::string& str )
+{
+	errno = 0;
+	const char* stringToConvert = str.c_str();
+	const char* endOfConversion = stringToConvert;
+	double value = STRTOD( stringToConvert, const_cast<char**>(&endOfConversion) );
+	if( value == 0.0 && stringToConvert == endOfConversion ) // error: conversion could not be performed
+		throw bad_lexical_cast(); // not a double
+	if (*endOfConversion)
+		throw bad_lexical_cast(); // started out as a double but became something else - like "10foo.raw"
+	return value;
+}
 
 /// Parses command line arguments to setup execution parameters, and
 /// contructs help text. Inputs are the arguments to main. A filled
 /// Config object is returned.
-Config parseCommandLine(int argc, const char* argv[])
+Config parseCommandLine(int argc, char** argv)
 {
     namespace po = boost::program_options;
 
@@ -166,6 +182,18 @@ Config parseCommandLine(int argc, const char* argv[])
     bool noindex = false;
     bool zlib = false;
     bool gzip = false;
+    bool ms_numpress_all = false; // if true, use this numpress compression with default tolerance
+    double ms_numpress_linear = -1; // if >= 0, use this numpress linear compression with this tolerance
+	std::string ms_numpress_linear_str; // input as text, to help with the "msconvert --numpresslinear foo.raw" case
+    stringstream ss;
+    ss << boost::format("%4.2g") % BinaryDataEncoder_default_numpressLinearErrorTolerance;
+    std::string ms_numpress_linear_default = ss.str();
+    bool ms_numpress_pic = false; // if true, use this numpress Pic compression
+    double ms_numpress_slof = -1; // if >= 0, use this numpress slof compression with this tolerance
+	std::string ms_numpress_slof_str; // input as text, to help with the "msconvert --numpressslof foo.raw" case
+    stringstream ss2;
+    ss2 << boost::format("%4.2g") % BinaryDataEncoder_default_numpressSlofErrorTolerance;
+    std::string ms_numpress_slof_default = ss2.str();
     bool detailedHelp = false;
 
     po::options_description od_config("Options");
@@ -248,6 +276,18 @@ Config parseCommandLine(int argc, const char* argv[])
         ("zlib,z",
             po::value<bool>(&zlib)->zero_tokens(),
             ": use zlib compression for binary data")
+        ("numpressLinear",
+            po::value<std::string>(&ms_numpress_linear_str)->implicit_value(ms_numpress_linear_default),
+            ": use numpress linear prediction compression for binary mz and rt data (relative accuracy loss will not exceed given tolerance arg, unless set to 0)")
+        ("numpressPic",
+            po::value<bool>(&ms_numpress_pic)->zero_tokens(),
+            ": use numpress positive integer compression for binary intensities (absolute accuracy loss will not exceed 0.5)")
+        ("numpressSlof",
+            po::value<std::string>(&ms_numpress_slof_str)->implicit_value(ms_numpress_slof_default),
+            ": use numpress short logged float compression for binary intensities (relative accuracy loss will not exceed given tolerance arg, unless set to 0)")
+        ("numpressAll,n",
+            po::value<bool>(&ms_numpress_all)->zero_tokens(),
+            ": same as --numpressLinear --numpressSlof (see https://github.com/fickludd/ms-numpress for more info)")
         ("gzip,g",
             po::value<bool>(&gzip)->zero_tokens(),
             ": gzip entire output file (adds .gz to filename)")
@@ -263,6 +303,12 @@ Config parseCommandLine(int argc, const char* argv[])
         ("srmAsSpectra",
             po::value<bool>(&config.srmAsSpectra)->zero_tokens(),
             ": write selected reaction monitoring as spectra, not chromatograms")
+        ("combineIonMobilitySpectra",
+            po::value<bool>(&config.combineIonMobilitySpectra)->zero_tokens(),
+            ": write all drift bins/scans in a frame/block as one spectrum instead of individual spectra")
+        ("acceptZeroLengthSpectra",
+            po::value<bool>(&config.acceptZeroLengthSpectra)->zero_tokens(),
+            ": some vendor readers have an efficient way of filtering out empty spectra, but it takes more time to open the file")
         ("help",
             po::value<bool>(&detailedHelp)->zero_tokens(),
             ": show this message, with extra detail on filter options")
@@ -424,6 +470,29 @@ Config parseCommandLine(int argc, const char* argv[])
 
     // check stuff
 
+	if (ms_numpress_slof_str.length()) // was that a numerical arg to --numpressSlof, or a filename?
+	{
+		try 
+		{
+			ms_numpress_slof = string_to_double(ms_numpress_slof_str); 
+		}
+		catch(...) {
+			config.filenames.push_back(ms_numpress_slof_str); // actually that was a filename
+            ms_numpress_slof = BinaryDataEncoder_default_numpressSlofErrorTolerance;
+		}
+	}
+	if (ms_numpress_linear_str.length()) // was that a numerical arg to --numpressLinear, or a filename?
+	{
+		try 
+		{
+			ms_numpress_linear = string_to_double(ms_numpress_linear_str); 
+		}
+		catch(...) {
+			config.filenames.push_back(ms_numpress_linear_str); // actually that was a filename
+            ms_numpress_linear = BinaryDataEncoder_default_numpressLinearErrorTolerance;
+		}
+	}
+
     if (config.filenames.empty())
         throw user_error("[msconvert] No files specified.");
 
@@ -529,6 +598,35 @@ Config parseCommandLine(int argc, const char* argv[])
     if (zlib)
         config.writeConfig.binaryDataEncoderConfig.compression = BinaryDataEncoder::Compression_Zlib;
 
+    if ((ms_numpress_slof>=0) && ms_numpress_pic)
+        throw user_error("[msconvert] Incompatible compression flags 'numpressPic' and 'numpressSlof'.");
+
+    if (ms_numpress_all && ms_numpress_pic)
+        throw user_error("[msconvert] Incompatible compression flags 'numpressPic' and 'numpressAll'.");
+
+    if (ms_numpress_all) {
+        config.writeConfig.binaryDataEncoderConfig.numpressOverrides[MS_m_z_array] = BinaryDataEncoder::Numpress_Linear;
+        config.writeConfig.binaryDataEncoderConfig.numpressOverrides[MS_time_array] = BinaryDataEncoder::Numpress_Linear;
+        config.writeConfig.binaryDataEncoderConfig.numpressOverrides[MS_intensity_array] = BinaryDataEncoder::Numpress_Slof;
+        config.writeConfig.binaryDataEncoderConfig.numpressLinearErrorTolerance = BinaryDataEncoder_default_numpressLinearErrorTolerance;
+        config.writeConfig.binaryDataEncoderConfig.numpressSlofErrorTolerance = BinaryDataEncoder_default_numpressSlofErrorTolerance;
+    }
+    if (ms_numpress_pic) 
+    {
+        config.writeConfig.binaryDataEncoderConfig.numpressOverrides[MS_intensity_array] = BinaryDataEncoder::Numpress_Pic;
+    }    
+    if (ms_numpress_slof>=0) 
+    {
+        config.writeConfig.binaryDataEncoderConfig.numpressOverrides[MS_intensity_array] = BinaryDataEncoder::Numpress_Slof;
+        config.writeConfig.binaryDataEncoderConfig.numpressSlofErrorTolerance = ms_numpress_slof;
+    }
+    if (ms_numpress_linear>=0) 
+    {
+        config.writeConfig.binaryDataEncoderConfig.numpressOverrides[MS_m_z_array] = BinaryDataEncoder::Numpress_Linear;
+        config.writeConfig.binaryDataEncoderConfig.numpressOverrides[MS_time_array] = BinaryDataEncoder::Numpress_Linear;
+        config.writeConfig.binaryDataEncoderConfig.numpressLinearErrorTolerance = ms_numpress_linear;
+    }
+
     return config;
 }
 
@@ -578,9 +676,7 @@ int mergeFiles(const vector<string>& filenames, const Config& config, const Read
     vector<MSDataPtr> msdList;
     int failedFileCount = 0;
 
-    ReaderList::Config readerConfig;
-    readerConfig.simAsSpectra = config.simAsSpectra;
-    readerConfig.srmAsSpectra = config.srmAsSpectra;
+    ReaderList::Config readerConfig(config);
 
     // Each file is read in separately in MSData objects in the msdList list.
     BOOST_FOREACH(const string& filename, filenames)
@@ -643,9 +739,7 @@ void processFile(const string& filename, const Config& config, const ReaderList&
 
     *os_ << "processing file: " << filename << endl;
 
-    ReaderList::Config readerConfig;
-    readerConfig.simAsSpectra = config.simAsSpectra;
-    readerConfig.srmAsSpectra = config.srmAsSpectra;
+    ReaderList::Config readerConfig(config);
 
     vector<MSDataPtr> msdList;
     readers.read(filename, msdList, readerConfig);
@@ -729,8 +823,14 @@ int go(const Config& config)
 }
 
 
-int main(int argc, const char* argv[])
+int main(int argc, char* argv[])
 {
+    bnw::args args(argc, argv);
+
+    std::locale global_loc = std::locale();
+    std::locale loc(global_loc, new bfs::detail::utf8_codecvt_facet);
+    bfs::path::imbue(loc);
+
     try
     {
         Config config = parseCommandLine(argc, argv);
